@@ -1,5 +1,6 @@
 """Actions that can be performed on instances and products.
 
+
  Everything that is handy to have as a single unit of action is turned
  into an Action. They can be quite small (just stopping zope), but
  that's OK. Being able to say 'instancemanager myproject stop' from
@@ -84,120 +85,10 @@ class CreateInstanceAction(Action):
     option = '--create'
 
     def run(self, options=None):
-        useZeo = self.conf.configData['use_zeo']
-        python = self.conf.configData['python']
-        instanceDir = self.conf.instanceDir()
-        zeoDir = self.conf.zeoDir()
-        dirs_to_remove = [instanceDir]
-        if useZeo:
-            dirs_to_remove.append(zeoDir)
-        for dir in dirs_to_remove:
-            try:
-                actionutils.trapUsageWithinVolatileDir(dir)
-            except InvokedInVolatileDir, error:
-                log.error(error)
-                sys.exit(1)
-            if os.path.exists(dir):
-                shutil.rmtree(dir)
-                log.debug("Removed instance/zeo dir '%s'.", dir)
-        # Zope server
-        mkInstance = os.path.join(self.conf.zopeDir(),
-                                  'bin',
-                                  'mkzopeinstance.py')
-        params = '-u %s:%s -d %s' % (
-            self.conf.configData['user'],
-            self.conf.configData['password'],
-            instanceDir,
-            )
-        command = ' '.join([python, mkInstance, params])
-        os.system(command)
-        log.info("Created zope instance in '%s'.",
-                 self.conf.instanceDir())
-        self.handleZopeConf(useZeo=useZeo)
-        # Zeo server
-        if useZeo:
-            mkInstance = os.path.join(self.conf.zopeDir(),
-                                      'bin',
-                                      'mkzeoinstance.py')
-            params = '%s %s' % (
-                zeoDir,
-                self.conf.zeoPort(),
-                )
-            command = ' '.join([python, mkInstance, params])
-            os.system(command)
-            log.info("Created zeo instance in '%s'.",
-                     self.conf.zeoDir())
-            zeoconfigFile = os.path.join(zeoDir, 'etc', 'zeo.conf')
-            premadeZeoconf = self.conf.zeoconf()
-            if os.path.exists(premadeZeoconf):
-                source = premadeZeoconf
-                target = zeoconfigFile
-                shutil.copy(source, target)
-                log.info("Copied over premade zeo.conf from %s.", source)
-                log.debug("Copied it to %s.", target)
-                return
-            log.debug("Replacing parts of '%s'.", zeoconfigFile)
-            oldFile = open(zeoconfigFile, 'r').readlines()
-            newFile = open(zeoconfigFile, 'w')
-            textsToChange = config.ZEOCONFIGCHANGES.keys()
-            for line in oldFile:
-                newLine = line
-                for text in textsToChange:
-                    if text in line:
-                        template = config.ZEOCONFIGCHANGES[text]
-                        newLine = template % self.conf.configData
-                        log.debug("Replaced %r", line)
-                        log.debug("....with %r", newLine)
-                        textsToChange.remove(text)
-                newFile.write(newLine)
-            newFile.close()
-            log.info("Changed the zeo config.")
-                    
-    def handleZopeConf(self, useZeo=False):
-        instanceDir = self.conf.instanceDir()
-        zopeConfigFile = os.path.join(instanceDir, 'etc', 'zope.conf')
-        premadeZopeconf = self.conf.zopeconf()
-        if os.path.exists(premadeZopeconf):
-            source = premadeZopeconf
-            target = zopeConfigFile
-            shutil.copy(source, target)
-            log.info("Copied over premade zope.conf from %s.", source)
-            log.debug("Copied it to %s.", target)
-            return
-        log.debug("Replacing parts of '%s'.", zopeConfigFile)
-        oldFile = open(zopeConfigFile, 'r').readlines()
-        newFile = open(zopeConfigFile, 'w')
-        textsToChange = config.CONFIGCHANGES.keys()
-        for line in oldFile:
-            newLine = line
-            for text in textsToChange:
-                if text in line:
-                    template = config.CONFIGCHANGES[text]
-                    newLine = template % self.conf.configData
-                    log.debug("Replaced %r", line)
-                    log.debug("....with %r", newLine)
-                    textsToChange.remove(text)
-            newFile.write(newLine)
-        newFile.close()
-        if useZeo:
-            oldFile = open(zopeConfigFile, 'r').readlines()
-            newFile = open(zopeConfigFile, 'w')
-            skip = False
-            SKIPSTART = '<zodb_db main>'
-            SKIPEND = '</zodb_db>'
-            for line in oldFile:
-                newLine = line
-                if line.startswith(SKIPSTART):
-                    skip = True
-                if not skip:
-                    newFile.write(newLine)
-                if line.startswith(SKIPEND):
-                    skip = False
-            newFile.write(config.ZEOSNIPPET % {
-                    'zeoport': self.conf.zeoPort()
-                    })
-        log.info("Changed the zope config.")
-        
+        # We want to reuse this method in another action
+        #
+        actionutils.createInstance(self.conf, purge=True)
+
 
 class CopyDatafsAction(Action):
     """Copy over a fresh, prepared, Data.fs.
@@ -224,8 +115,8 @@ class CopyDatafsAction(Action):
                      "skipping this step.", source)
             return
         else:
+            log.info("Copying over fresh Data.fs from %s.", source)
             shutil.copy(source, target)
-            log.info("Copied over fresh Data.fs from %s.", source)
             log.debug("Copied it to %s.", target)
 
 class AddProductsAction(Action):
@@ -449,15 +340,91 @@ class QuickreinstallAction(Action):
         # Second argument: id of admin user who creates the plone site
         arguments.append(adminUser)
         # Next the list of main products
-        arguments += self.conf.configData['main_products']
-        arguments += self.conf.configData['generic_setup_profiles']
-        #arguments.append(u'profile-Products.mauritssetup:mauritssetup')
+        arguments += actionutils.adaptListForDevelopment( self.conf,
+            self.conf.configData['main_products'])
+        # and the GenericSetup profiles
+        arguments += actionutils.adaptListForDevelopment( self.conf,
+            self.conf.configData['generic_setup_profiles'])
 
         # Now prepare it for the BaseZopeAction.
         command = ' '.join(arguments)
         actionutils.runZopectl(self.conf, command)
         log.info("Quick reinstall script called.")
 
+class UninstallAction(ArgumentAction):
+    """Action that uninstalls a given product from your instance.
+    """
+
+    name = "Action that uninstalls a given product"
+    option = '--uninstall'
+    alternativeOption = '-u'
+
+    def scriptLocation(self):
+        """Return the location of the uninstall script.
+        """
+
+        ourDir = os.path.dirname(os.path.abspath(__file__))
+        location = os.path.join(ourDir, config.UISCRIPT)
+        log.debug("Uninstall script location is %s.", location)
+        return location
+
+    def run(self, options=None):
+        product = options.uninstall
+        ploneSiteName = self.conf.configData['plone_site_name']
+        adminUser = self.conf.configData['user']
+        if not ploneSiteName:
+            log.debug("No plone site name, so no uninstall.")
+            log.info("No uninstall wanted.")
+            return
+        uninstallScript = self.scriptLocation()
+        arguments = []
+        arguments.append('run') # Run a script.
+        arguments.append(uninstallScript) # The script.
+        # First argument: plone site name
+        arguments.append(ploneSiteName)
+        # Second argument: id of admin user who creates the plone site
+        arguments.append(adminUser)
+        # Next the product
+        arguments.append(product)
+
+        # Now prepare it for the BaseZopeAction.
+        command = ' '.join(arguments)
+        actionutils.runZopectl(self.conf, command)
+        log.info("Quick uninstall script called.")
+
+
+class UpgradeZopeAction(Action):
+    """ Copies over the zope.conf or regenerates it based on the current
+        configuration and copies over the zopectl from the correct skeleton.
+        Upgrade your instance in 2 steps:
+        
+        1. Update the Zope version in your project.py
+        2. run instancemanager --upgradezope
+    """
+    
+    name = "Update the zope.conf used for your instance"
+    option = '--upgradezope'
+    
+    def run(self, options=None):
+        # update the executables by deleting them from the instance and then
+        # running mkzopeinstance.py again. Also zope.conf is refreshed
+        useZeo = self.conf.configData['use_zeo']
+        python = self.conf.configData['python']
+        instanceDir = self.conf.instanceDir()
+        filesnames = ['zopectl', 'runzope', 'zopeservice.py', 'runzope.bat']
+        files = [os.path.join(instanceDir, 'bin', name) for name in filesnames]
+        files.append(os.path.join(instanceDir, 'etc','zope.conf'))
+        if useZeo:
+            zeoDir = self.conf.zeoDir()
+            files.append(os.path.join(zeoDir, 'bin', 'zeoctl'))
+            files.append(os.path.join(zeoDir, 'bin', 'runzeo'))
+            files.append(os.path.join(zeoDir, 'etc', 'zeo.conf'))
+        for f in files:
+            if os.path.exists(f):
+                shutil.os.remove(f)
+                log.debug("Removed %s" % f)
+        actionutils.createInstance(self.conf, purge=False)
+
 
 class PackAction(Action):
     """Action that packs your database.
