@@ -69,6 +69,8 @@ class Configuration(object):
 
         * Our own defaults (defaults.py).
 
+        * Site-wide defaults (/etc/instancemanager/sitedefaults.py).
+
         * User defaults (userdefaults.py in the config dir).
 
         * Per-project extra configuration (projectname.py in the
@@ -76,6 +78,11 @@ class Configuration(object):
 
        The variables imported from these files should be placed in
         self.configData.  
+
+        For the user defaults and the project configuration, you can
+        add a file named like the defaults/project config, but
+        prefixed with 'local-'. That's handy for usernames and
+        passwords that you don't want in subversion, for instance.
         """
 
         log.debug("Loading our configuration defaults.")
@@ -85,17 +92,41 @@ class Configuration(object):
         self.configData['project'] = project
         import defaults
         self._collectAttributes(defaults)
+        # Second: site-wide defaults.
+        siteDefaults = self._loadUserConfigModule(
+            config.SITECONFIG)
+        if siteDefaults:
+            self._collectAttributes(siteDefaults)
+        else:
+            log.debug("No site-wide defaults file.")
+        # Third, user defaults (including secret).
         userDefaults = self._loadUserConfigModule('userdefaults')
-        self._collectAttributes(userDefaults)
-        secretUserDefaults = self._loadUserConfigModule(config.SECRET_PREFIX+'userdefaults')
+        if userDefaults:
+            self._collectAttributes(userDefaults)
+        else:
+            log.critical("User defaults could not be read from your "
+                         "config directory.")
+            sys.exit(1)
+        secretUserDefaults = self._loadUserConfigModule(
+            config.SECRET_PREFIX+'userdefaults')
         if secretUserDefaults:
             self._collectAttributes(secretUserDefaults)
+        else:
+            log.debug("No secret user defaults file.")
         if project:
+            # Fourth: project defaults (including secret).
             projectDefaults = self._loadUserConfigModule(project)
-            self._collectAttributes(projectDefaults)
-            secretSettings = self._loadUserConfigModule(config.SECRET_PREFIX+project)
-            if secretSettings:
-                self._collectAttributes(secretSettings)
+            if projectDefaults:
+                self._collectAttributes(projectDefaults)
+            else:
+                log.warn("Project config file for %s not found "
+                         "in your config directory.", project)
+            secretProjectDefaults = self._loadUserConfigModule(
+                config.SECRET_PREFIX+project)
+            if secretProjectDefaults:
+                self._collectAttributes(secretProjectDefaults)
+            else:
+                log.debug("No secret project config file.")
             
     def _fixupRenamedAttributes(self,attribute):
         return renamedAttrs.get(attribute, attribute)
@@ -114,40 +145,26 @@ class Configuration(object):
             tempDict[dattribute] = getattr(module, attribute)
         self.configData.update(tempDict)
         log.debug("configData is now %r.", self.configData)
-    oneshotMsgs={'defaults': (
-                    "You may wish to put local information common to\n"
-                    " all projects, such as user and password, or local\n"
-                    " directories into a file named %s.py"),
-                 'project':(
-                    "You may wish to put local information for this\n"
-                    " project, such as user and password, into a file\n"
-                    " named %s.py")
-                }
-    def _loadUserConfigModule(self, name):
+
+    def _loadUserConfigModule(self, name, absolutePath=None):
         """Return module loaded from the user's config dir.
+
+        If an absolute path is specified, use that as the filename.
         """
-        
-        log.debug("Trying to load user config module '%s'.", name)
-        moduleFile = os.path.join(self.configDir, name+'.py')
-        try:
-            module = imp.load_source(name, moduleFile)
-        except IOError:
-            if name==config.SECRET_PREFIX+'userdefaults':
-                msg=self.oneshotMsgs['defaults']
-                if msg: 
-                    msg=msg % name
-                    self.oneshotMsgs['defaults'] = None
-                    log.info(msg)
-            elif name.startswith(config.SECRET_PREFIX):
-                msg=self.oneshotMsgs['project']
-                if msg: 
-                    msg=msg % name
-                    self.oneshotMsgs['project'] = None
-                    log.info(msg)
-            else:
-                msg="Couldn't find '%s' in user config dir.", name
-                log.warn(msg)
+
+        if absolutePath:
+            moduleFile = absolutePath
+            log.debug("Trying to load config file at '%s'.",
+                      moduleFile)
+        else:
+            log.debug("Trying to load user config module '%s'.", name)
+            moduleFile = os.path.join(self.configDir, name+'.py')
+        # Check if the file exists
+        if not os.path.exists(moduleFile):
+            log.debug("Couldn't find '%s'.", moduleFile)
             return None
+        # Import it.
+        module = imp.load_source(name, moduleFile)
         return module
 
     def expandTemplate(self, templateName):
