@@ -51,6 +51,8 @@ zope_location_template = '/opt/zope/zope%(zope_version)s'
 user = 'test'
 password = 'test'
 port = '8080'
+#ftp_port = '8021'
+ftp_port = None
 
 # Data for the zeo server.
 # If use_zeo is False, there will just be one zope server. If it is
@@ -205,6 +207,7 @@ generic_setup_profiles= []
 # XXX Note for instancemanager developers: "multi_actions" are not
 # copied correctly at the moment by the
 # copy_defaults_to_userdefaults.py script.
+# So remove those by hand afterwards.
 multi_actions = {
     # Fresh assumes a zeo setup.
     'fresh': ['--zope stop',
