@@ -14,6 +14,7 @@ The basis has been a script originally made by Maurits van Rees.
 """
 
 import sys
+import os
 
 try:
     from AccessControl.SecurityManagement import newSecurityManager
@@ -28,8 +29,21 @@ from AccessControl.SecurityManager import setSecurityPolicy
 from Products.CMFCore.tests.base.security import OmnipotentUser
 from Products.CMFCore.tests.base.security import PermissiveSecurityPolicy
 from Testing.makerequest import makerequest
-import transaction
 from Acquisition import Implicit
+try:
+    import transaction
+except:
+    # zope 2.7 BBB
+    import ZODB
+    transaction = False
+
+def transaction_commit():
+    if transaction:
+        transaction.commit()
+    else:
+        # zope 2.7 BBB
+        get_transaction().commit()
+    
 
 class OmnipotentUser( Implicit ):
     """
@@ -63,36 +77,12 @@ class OmnipotentUser( Implicit ):
     def _check_context(self, object):
         return True
 
-class FakeRequest:
-    """Support class needed for running reinstall for some products
-    (like cmflinkchecker, that requires a REQUEST for the catalog
-    updates it does).
+    def has_permission(self, *args, **kwargs):
+        return True
 
-    Lifted from Rafael Ritz's
-    http://lists.plone.org/pipermail/setup/2005-November/000172.html
-    """
- 
-    def __init__(self):
-        self.form = {}
-        self.cookies = {}
-        
-    def get(self, key, default=None):
-        return self.form.get(key, default)
+    def has_role(self, *args, **kwargs):
+        return True
 
-    def set(self, key, value):
-        self.form[key] = value
-
-    def __getitem__(self, key):
-        return self.get(key)
-
-    def _hold(self, data):
-        pass
-    
-    def __len__(self):
-        return 0
-
-    def getPresentationSkin(self):
-        return None
 
 class FakeResponse:
     """Like the FakeRequest class above.  Needed for at least one of
@@ -118,6 +108,7 @@ class FakeResponse:
     def setHeader(self, *args): pass
     def setStatus(self, status): self.status = status
 
+
 class Reinstaller:
 
     def __init__(self, app):
@@ -153,28 +144,25 @@ class Reinstaller:
         self.determineReinstallableProducts()
         self.reinstall()
         self.printErrors()
-        transaction.commit()
+        transaction_commit()
 
     def ensurePloneSiteExists(self):
         """Ensure that the Plone Site actually exists.  Create it if
         does not.
         """
 
-        if not hasattr(app, self.ploneSiteName):
+        if not hasattr(self.app, self.ploneSiteName):
             print 'INFO: Plone Site %s does not yet exist. Creating...' \
                   % self.ploneSiteName
-            if hasattr(app.manage_addProduct['CMFPlone'], 'addPloneSite'):
+            if hasattr(self.app.manage_addProduct['CMFPlone'], 'addPloneSite'):
                 # plone 2.5
-                factory = app.manage_addProduct['CMFPlone'].addPloneSite
+                factory = self.app.manage_addProduct['CMFPlone'].addPloneSite
             else:
                 # plone 2.1
-                factory = app.manage_addProduct['CMFPlone'].manage_addSite
-            request = FakeRequest()
-            self.app.REQUEST = request
+                factory = self.app.manage_addProduct['CMFPlone'].manage_addSite
             factory(self.ploneSiteName, self.ploneSiteName)
-            del self.app.REQUEST
-            transaction.commit()
-        self.ploneSite = getattr(app, self.ploneSiteName, None)
+            transaction_commit()
+        self.ploneSite = getattr(self.app, self.ploneSiteName, None)
         print 'INFO: %s' % self.ploneSite
         self.qi = self.ploneSite['portal_quickinstaller']
 
@@ -199,46 +187,96 @@ class Reinstaller:
         portal_setup = self.ploneSite.portal_setup
         for profile in self.profiles:
             print "Running GenericSetup for %s..." % profile
-            request = FakeRequest()
-            self.app.REQUEST = request
             try:
                 portal_setup.setImportContext(profile)
                 portal_setup.runAllImportSteps()
             except:
                 print "ERROR: Running GenericSetup for %s failed." % profile
                 self.errors += 1
-            del self.app.REQUEST
-            transaction.commit()
+            transaction_commit()
 
     def handleMigration(self):
         """See if plone itself needs migrating and do it if needed.
         """
-        
+
+        # In case of a upgrade from plone 2.0.5 to plone-2.1 with ATCT allready
+        # installed, first reinstall ATCT before running migration.
+        print "Determining if ATContentTypes needs to be reinstalled..."
+        atct = 'ATContentTypes'
+        self.qi = self.ploneSite['portal_quickinstaller']
+        atct_installed = self.qi.isProductInstalled(atct)
+        atct_tool = getattr(self.ploneSite, 'portal_atct', None)
+        fs_version = self.qi.getProductVersion(atct)
+        if (atct_installed and not atct_tool 
+            and fs_version > '0.2.0-final (CVS/Unreleased)'):
+            print "ATContentTypes needs reinstall before upgrading Plone"
+	    self.qi.reinstallProducts([atct])
+            transaction_commit()
+            print "Reinstalled of ATContentTypes succeeded."
+        else:
+	    print "Reinstall of ATContentTypes is not needed."
+
         print "Determining if plone needs migrating..."
         pm = self.ploneSite['portal_migration']
-        if pm.needUpgrading():
-            request = FakeRequest()
-            self.app.REQUEST = request
-            print "Plone itself needs upgrading!"
+        ploneVersion = pm.getInstanceVersion() # 2.5.2
+        if '-' not in ploneVersion:
+            ploneVersion = ploneVersion + '-zzz'
+        availableVersions = []
+        for version in pm.knownVersions():
+            if '-' not in version:
+                version = version + '-zzz'
+            availableVersions.append(version)
+        availableVersions.sort()
+        latestVersion = availableVersions[-1]
+        if ploneVersion != latestVersion:
+            # migration needed.
+            # pm.needUpgrading() also wants to update svn versions.
+            print "Plone itself needs upgrading! (%s to %s)" %\
+                (ploneVersion, latestVersion)
             pm.upgrade(swallow_errors=1)
-            del self.app.REQUEST
-            transaction.commit()
+            # And now to prevent (svn/dev) versions:
+            latestVersion = latestVersion.replace('-zzz', '')
+            pm.setInstanceVersion(latestVersion)
+            transaction_commit()
             print "Migrated plone to latest version."
         else:
             print "Plone doesn't need to be migrated."
 
         print "Determining if ATContentTypes needs migrating..."
-        atct_tool = self.ploneSite['portal_atct']
-        if atct_tool.needsVersionMigration():
-            request = FakeRequest()
-            self.app.REQUEST = request
-            print "ATContentTypes needs upgrading!"
-            atct_tool.upgrade(swallow_errors=1)
-            del self.app.REQUEST
-            transaction.commit()
-            print "Migrated ATContentTypes to latest version."
-        else:
-            print "ATContentTypes doesn't need to be migrated."
+        atct_tool = getattr(self.ploneSite, 'portal_atct', None)
+        if atct_tool:
+            atctVersion = atct_tool.getVersion()[1] # [1] gives us the string.
+            if ' ' in atctVersion:
+                # Some "2.3.4 (svn/dev)" version, we'll just clean that
+                # here to get an easier update.
+                parts = atctVersion.split(' ')
+                version = parts[0]
+                version = version.replace('-devel', '-final')
+                atct_tool.setInstanceVersion(version)
+                atctVersion = version
+            atctVersion = atctVersion.replace('-final', '-zzz')
+            availableVersions = []
+            for version in atct_tool.knownVersions():
+                if '-' not in version:
+                    version = version + '-zzz'
+                version = version.replace('-final', '-zzz')
+                availableVersions.append(version)
+            availableVersions.sort()
+            latestVersion = availableVersions[-1]
+            if atctVersion < latestVersion:
+                # ^^^ '<' instead of '!=' as somehow the atct version
+                # can end up higher than the latest version...
+                print "ATContentTypes needs upgrading! (%s to %s)"%\
+                    (atctVersion, latestVersion)
+                atct_tool.upgrade(swallow_errors=1)
+                transaction_commit()
+                # Now to keep the (svn/dev) versions out of there:
+                latestVersion = latestVersion.replace('-zzz',
+                                                      '-final')
+                atct_tool.setInstanceVersion(latestVersion)
+                print "Migrated ATContentTypes to latest version."
+            else:
+                print "ATContentTypes doesn't need to be migrated."
 
     def determineReinstallableProducts(self):
         """Determine which products need to be reinstalled.
@@ -282,8 +320,6 @@ class Reinstaller:
         print "Reinstalling..."
         for productId in self.reinstallableProducts:
             try:
-                request = FakeRequest()
-                self.app.REQUEST = request
                 response = FakeResponse()
                 self.app.RESPONSE = response
                 if self.qi.isProductInstalled(productId):
@@ -295,12 +331,14 @@ class Reinstaller:
                     self.qi.installProduct(productId)
                     print "Installed %s (so no REinstall)." % \
                         productId
-                del self.app.REQUEST
-                del self.app.RESPONSE
             except:
                 print "ERROR: reinstall failed for %s!" % productId
                 self.errors += 1
-                del self.app.REQUEST
+            try:
+                del self.app.RESPONSE
+            except:
+                pass
+    
 
     def printErrors(self):
         """Print feedback about errors/warnings.
