@@ -2,6 +2,7 @@ import config
 import logging
 import os
 import os.path
+import shutil
 import sys
 import popen2
 
@@ -44,17 +45,17 @@ def addConsoleLogging(level=logging.INFO):
     hdlr.setLevel(level)
     hdlr.setFormatter(formatter)
     log.addHandler(hdlr)
-    
+
 def adaptiveUnarchive(source):
     """Untar or unzip a file to the current directory.
        TODO: is using "not in gzip format" an i18n issue?
-       NOTE: using extensions to decide whether a file is tar or tgz format 
+       NOTE: using extensions to decide whether a file is tar or tgz format
          is very unreliable for Plone product archives.
-       We do not test for or return overall success. 
+       We do not test for or return overall success.
        The next step in the operation will look for unpacked items.
     """
     tryAgain=False
-    
+
     if source.endswith('zip'):
         os.system("unzip -qq %s" % source)
     else:
@@ -73,10 +74,21 @@ def symlink(source, target, conf):
     useSvnExport = conf.configData['use_svn_export']
     if not useSvnExport:
         # Normal behaviour.
-        log.debug("Symlinking %s -> %s.", source, target)
-        os.symlink(source, target)
+        if conf.configData['is_windows']:
+            # windows must use copy instead of symlink
+            log.debug("Copying %s -> %s.", source, target)
+            shutil.copytree(source, target)
+        else:
+            log.debug("Symlinking %s -> %s.", source, target)
+            os.symlink(source, target)
     else:
         # Svn export is needed
         command = ' '.join(['svn export', '-q', source, target])
         log.debug("Svn exporting: %s.", command)
         os.system(command)
+
+def isZope3(conf):
+    zope_version = conf.configData['zope_version']
+    if zope_version[0] == '3':
+        return True
+    return False
\ No newline at end of file
